@@ -56,6 +56,16 @@ class random_depictor:
             superatoms = superatoms.readlines()
             self.superatoms = [s[:-2] for s in superatoms]
 
+        # Define PIL resizing methods to choose from:
+        self.PIL_resize_methods = [
+        Image.NEAREST, 
+        Image.BOX, 
+        Image.BILINEAR, 
+        Image.HAMMING, 
+        Image.BICUBIC, 
+        Image.LANCZOS
+        ]
+
         # Set context for multiprocessing but make sure this only happens once
         try:
             set_start_method("spawn")
@@ -66,7 +76,7 @@ class random_depictor:
         self,
         smiles: str,
         shape: Tuple[int, int, int] = (299, 299),
-        grayscale: bool = True,
+        grayscale: bool = False,
     ):
         # Depict structure with random parameters
         depiction = self.random_depiction(smiles, shape)
@@ -201,7 +211,7 @@ class random_depictor:
         temp = renderer.renderToBuffer(molecule)
         temp = io.BytesIO(temp)
         depiction = sk_io.imread(temp) 
-        depiction = resize(depiction, (shape[0], shape[1], 4))
+        depiction = self.resize(depiction, (shape[0], shape[1]))
         depiction = rgba2rgb(depiction)
         depiction = img_as_ubyte(depiction)
         return depiction
@@ -268,7 +278,7 @@ class random_depictor:
             depiction_settings = self.get_random_rdkit_rendering_settings()
             # Create depiction
             # TODO: Figure out how to depict molecules without kekulization here.
-            # This does not prevent the molecule from being depicted kekulized:
+            # The following line does not prevent the molecule from being depicted kekulized:
             # mol = rdMolDraw2D.PrepareMolForDrawing(mol, kekulize = False)
             # The molecule must get kekulized somewhere "by accident"
 
@@ -276,7 +286,7 @@ class random_depictor:
             depiction = depiction_settings.GetDrawingText()
             depiction = sk_io.imread(io.BytesIO(depiction))
             # Resize image to desired shape
-            depiction = resize(depiction, shape)
+            depiction = self.resize(depiction, shape)
             depiction = img_as_ubyte(depiction)
             return np.asarray(depiction)
         else: 
@@ -372,7 +382,7 @@ class random_depictor:
         # Right now, this is loaded every time when a structure is depicted. That seems inefficient.
         if self.random_choice([True, False]):
             cdk_superatom_abrv = JClass("org.openscience.cdk.depict.Abbreviations")()
-            abbreviation_path = str(HERE.joinpath("superatom_obabel.smi")).replace(
+            abbreviation_path = str(HERE.joinpath("smiles_list.smi")).replace(
                 "\\", "/"
             )
             abbreviation_path = JClass("java.lang.String")(abbreviation_path)
@@ -455,6 +465,13 @@ class random_depictor:
 
         # Create an empty image of the right size
         y, x = self.random_image_size(shape)
+        # Workaround for structures that are cut off at edged of images:
+        # Make image twice as big, reduce Zoom factor, then remove white 
+        # areas at borders and resize to originally desired shape
+        # TODO: Find out why the structures are cut off in the first place
+        y = y * 3
+        x = x * 3
+
         drawArea = JClass("java.awt.Rectangle")(x, y)
         BufferedImage = JClass("java.awt.image.BufferedImage")
         image = BufferedImage(x, y, BufferedImage.TYPE_INT_RGB)
@@ -471,7 +488,7 @@ class random_depictor:
             JClass(
                 "org.openscience.cdk.renderer.generators.BasicSceneGenerator.ZoomFactor"
             ),
-            double(0.9),
+            double(0.75),
         )
         g2 = image.getGraphics()
         g2.setColor(JClass("java.awt.Color").WHITE)
@@ -492,10 +509,53 @@ class random_depictor:
 
         # Read image in skimage
         depiction = sk_io.imread(depiction, plugin="imageio")
-        depiction = resize(depiction, shape)
+        # Normalise padding and get non-distorted image of right size
+        depiction = self.normalise_padding(depiction)
+        depiction = self.central_square_image(depiction)
+        depiction = self.resize(depiction, shape)
         depiction = img_as_ubyte(depiction)
         return depiction
 
+    def normalise_padding(self, im: np.array) -> np.array:
+        """This function takes an RGB image (np.array) and deletes white space at 
+        the borders. Then 0-10% of the image width/height is added as padding again.
+        The modified image is returned
+        ___
+        im: np.array
+        ___
+        output: the modified image (np.array)
+        """
+        mask = im > 200 
+        all_white = mask.sum(axis=2) > 0
+        rows = np.flatnonzero((~all_white).sum(axis=1))
+        cols = np.flatnonzero((~all_white).sum(axis=0))
+        crop = im[rows.min():rows.max()+1, cols.min():cols.max()+1, :]
+        pad = self.random_choice(np.arange(5, int(crop.shape[0]*0.2), 1))
+        crop = np.pad(crop, pad_width=((pad,pad), (pad,pad), (0,0)), mode='constant', constant_values=255)
+        return crop
+
+    def central_square_image(self, im: np.array) -> np.array:
+        """
+        This function takes image (np.array) and will add white padding 
+        so that the image has a square shape with the width/height of the longest side 
+        of the original image.
+        ___
+        im: np.array
+        ___
+        output: np.array
+        """
+        # Create new blank white image
+        max_wh = max(im.shape)
+        new_im = 255 * np.ones((max_wh, max_wh, 3), np.uint8)
+        # Determine paste coordinates and paste image
+        upper = int((new_im.shape[0]-im.shape[0])/2)
+        lower = int((new_im.shape[0]-im.shape[0])/2) + im.shape[0]
+        left = int((new_im.shape[1]-im.shape[1])/2)
+        right = int((new_im.shape[1]-im.shape[1])/2) + im.shape[1]
+        new_im[upper:lower, left:right] = im
+        return new_im
+
+
     def random_depiction(
         self, smiles: str, shape: Tuple[int, int] = (299, 299)
     ) -> np.array:
@@ -516,6 +576,28 @@ class random_depictor:
             depiction = depiction_function(smiles, shape)
         return depiction
 
+
+    def resize(
+        self, image: np.array, shape:Tuple[int]
+        ) -> np.array:
+        """
+        This function takes an image (np.array) and a shape and returns the resized image (np.array).
+        It uses Pillow to do this, as it seems to have a bigger variety of scaling methods than skimage.
+        The up/downscaling method is chosen randomly.
+        ___
+        image: np.array; the input image
+        shape: tuple that describes desired output shape
+        ___
+        Output: np.array; the resized image
+
+        """
+        image = Image.fromarray(image)
+        shape = (shape[0], shape[1])
+        image = image.resize(shape, resample = self.random_choice(self.PIL_resize_methods))
+        return np.asarray(image)
+
+
+
     def imgaug_augment(self, image: np.array) -> np.array:
         """This function applies a random amount of augmentations to a given image (np.array)
         using and returns the augmented image (np.array)."""
@@ -578,7 +660,8 @@ class random_depictor:
         selected_aug_list = self.random_choices(aug_list, aug_number)
         aug = iaa.Sequential(selected_aug_list)
         augmented_image = aug.augment_images([image])[0]
-        augmented_image = resize(augmented_image, original_shape)
+        augmented_image = self.resize(augmented_image, original_shape)
+        augmented_image = augmented_image.astype(np.uint8)
         return augmented_image
 
     def get_random_label_position(self, width: int, height: int) -> Tuple:
@@ -824,7 +907,7 @@ class random_depictor:
                 )
         return R_group_label
 
-    def add_chemical_label(self, image: np.array, label_type: str) -> np.array:
+    def add_chemical_label(self, image: np.array, label_type: str, foreign_fonts: bool= True) -> np.array:
         '''This function takes an image (np.array) and adds random text that looks like a chemical ID label ,
         an R group label or a reaction condition label around the structure. It returns the modified image.
         The label type is determined by the parameter label_type (str), which needs to be "ID", R_GROUP" or
@@ -833,7 +916,12 @@ class random_depictor:
         orig_image = deepcopy(im)
         width, height = im.size
         # Choose random font
-        font_dir = HERE.joinpath("fonts/")
+        if self.random_choice([True, False]) or not foreign_fonts:
+            font_dir = HERE.joinpath("fonts/")
+        #In half of the cases: Use foreign-looking font to generate bigger noise variety
+        else:
+            font_dir = HERE.joinpath("foreign_fonts/")
+
         fonts = os.listdir(str(font_dir))
         # Choose random font size
         font_sizes = range(10, 20)
@@ -862,7 +950,10 @@ class random_depictor:
                 (x_pos, y_pos), label_text, font=font
             )  # left, up, right, low
             paste_region = orig_image.crop(bounding_box)
-            mean = ImageStat.Stat(paste_region).mean
+            try:
+                mean = ImageStat.Stat(paste_region).mean
+            except ZeroDivisionError:
+                return np.asarray(im)
             if sum(mean) / len(mean) == 255:
                 draw.text((x_pos, y_pos), label_text, font=font, fill=(0, 0, 0, 255))
                 break
@@ -891,11 +982,10 @@ class random_depictor:
             new_arrow_image_shape = int(
                 (x_max - x_min) / self.random_choice(range(3, 6))
             ), int((y_max - y_min) / self.random_choice(range(3, 6)))
-            arrow_image = arrow_image.resize(
-                new_arrow_image_shape, resample=Image.BICUBIC
-            )
+            arrow_image = self.resize(np.asarray(arrow_image), new_arrow_image_shape)
+            arrow_image = Image.fromarray(arrow_image)
             arrow_image = arrow_image.rotate(
-                self.random_choice(range(360)), resample=Image.BICUBIC, expand=True
+                self.random_choice(range(360)), resample=self.random_choice([Image.BICUBIC, Image.NEAREST, Image.BILINEAR]), expand=True
             )
             # Try different positions with the condition that the arrows are overlapping with non-white pixels (the structure)
             for _ in range(50):
@@ -957,7 +1047,7 @@ class random_depictor:
             # Rotate completely randomly in half of the cases and in   180° steps in the other cases (higher probability that pasting works)
             if self.random_choice([True, False]):
                 arrow_image = arrow_image.rotate(
-                    self.random_choice(range(360)), resample=Image.BICUBIC, expand=True
+                    self.random_choice(range(360)), resample=self.random_choice([Image.BICUBIC, Image.NEAREST, Image.BILINEAR]), expand=True
                 )
             else:
                 arrow_image = arrow_image.rotate(self.random_choice([180, 360]))
@@ -1048,6 +1138,7 @@ class random_depictor:
         shape: Tuple[int, int] = (299, 299),
         ID_list: List[str] = False,
         processes: int = 4,
+        seed: int = 42,
     ) -> None:
         """This function takes a list of SMILES str, the amount of images to create per SMILES str and the path
         of an output directory. It then creates images_per_structure depictions of each chemical structure
@@ -1062,7 +1153,7 @@ class random_depictor:
                     output_dir,
                     shape,
                     ID_list[n],
-                    (n + 1) * len(smiles_list),  # individual seed
+                    (seed * n + 1) * len(smiles_list),  # individual seed
                 )
                 for n in range(len(smiles_list))
             )
@@ -1082,6 +1173,7 @@ class random_depictor:
         shape: Tuple[int, int] = (299, 299),
         ID_list: List[str] = False,
         processes: int = 4,
+        seed: int = 42,
     ) -> None:
         """This function takes a list of SMILES str, the amount of images to create per SMILES str and the path
         of an output directory. It then creates images_per_structure augmented depictions of each chemical structure
@@ -1096,7 +1188,7 @@ class random_depictor:
                     output_dir,
                     shape,
                     ID_list[n],
-                    (n + 1) * len(smiles_list),  # individual seed
+                    (seed * n + 1) * len(smiles_list),  # individual seed
                 )
                 for n in range(len(smiles_list))
             )
